@@ -29,10 +29,21 @@ class Schema(FancyValidator):
             name = None
     """
 
-    chained_validators = []
+    # These validators will be applied before this schema:
     pre_validators = []
+    # These validators will be applied after this schema:
+    chained_validators = []
+    # If true, then it is not an error when keys that aren't
+    # associated with a validator are present:
     allow_extra_fields = False
+    # If true, then keys that aren't associated with a validator
+    # are removed:
     filter_extra_fields = False
+    # If this is given, then any keys that aren't available but
+    # are expected  will be replaced with this value (and then
+    # validated!)  This does not override a present .if_missing
+    # attribute on validators:
+    if_key_missing = NoDefault
     compound = True
     fields = {}
     order = []
@@ -126,12 +137,32 @@ class Schema(FancyValidator):
                 except AttributeError:
                     if_missing = NoDefault
                 if if_missing is NoDefault:
-                    errors[name] = Invalid(
-                        self.message('missingValue', state),
-                        None, state)
+                    if self.if_key_missing is NoDefault:
+                        errors[name] = Invalid(
+                            self.message('missingValue', state),
+                            None, state)
+                    else:
+                        try:
+                            new[name] = validator.to_python(self.if_key_missing, state)
+                        except Invalid, e:
+                            errors[name] = e
                 else:
                     new[name] = validator.if_missing
 
+            if errors:
+                for validator in self.chained_validators:
+                    if (not hasattr(validator, 'validate_partial')
+                        or not getattr(validator, 'validate_partial_form', False)):
+                        continue
+                    try:
+                        validator.validate_partial(value_dict, state)
+                    except Invalid, e:
+                        sub_errors = e.unpack_errors()
+                        if not isinstance(sub_errors, dict):
+                            # Can't do anything here
+                            continue
+                        merge_dicts(errors, sub_errors)
+
             if errors:
                 raise Invalid(
                     format_compound_error(errors),
@@ -185,7 +216,7 @@ class Schema(FancyValidator):
             del __traceback_info__
 
             for name in unused:
-                validator = adapt_validator(self.fields[name], state)
+                validator = self.fields[name]
                 try:
                     new[name] = validator.from_python(None, state)
                 except Invalid, e:
@@ -264,3 +295,40 @@ def format_compound_error(v, indent=0):
     else:
         assert 0, "I didn't expect something like %s" % repr(v)
         
+def merge_dicts(d1, d2):
+    for key in d2:
+        if key in d1:
+            d1[key] = merge_values(d1[key], d2[key])
+        else:
+            d1[key] = d2[key]
+    return d1
+
+def merge_values(v1, v2):
+    if (isinstance(v1, (str, unicode))
+        and isinstance(v2, (str, unicode))):
+        return v1 + '\n' + v2
+    elif (isinstance(v1, (list, tuple))
+          and isinstance(v2, (list, tuple))):
+        return merge_lists(v1, v2)
+    elif isinstance(v1, dict) and isinstance(v2, dict):
+        return merge_dicts(v1, v2)
+    else:
+        # @@: Should we just ignore errors?  Seems we do...
+        return v1
+
+def merge_lists(l1, l2):
+    if len(l1) < len(l2):
+        l1 = l1 + [None]*(len(l2)-len(l1))
+    elif len(l2) < len(l1):
+        l2 = l2 + [None]*(len(l1)-len(l2))
+    result = []
+    for l1item, l2item in zip(l1, l2):
+        item = None
+        if l1item is None:
+            item = l2item
+        elif l2item is None:
+            item = l1item
+        else:
+            item = merge_values(l1item, l2item)
+        result.append(item)
+    return result
