@@ -24,14 +24,17 @@ Validator/Converters for use with FormEncode.
 """
 
 import re
-import cgi
 DateTime = None
 mxlookup = None
 httplib = None
 urlparse = None
 from interfaces import *
 from api import *
-from declarative import Declarative, DeclarativeMeta
+sha = random = None
+
+import cgi
+
+import fieldstorage
 
 True, False = (1==1), (0==1)
 
@@ -73,7 +76,7 @@ def datetime_now(module):
 
 def datetime_makedate(module, year, month, day):
     if module.__name__ == 'datetime':
-        return module.datetime.date(year, month, day)
+        return module.date(year, month, day)
     else:
         try:
             return module.DateTime(year, month, day)
@@ -83,7 +86,9 @@ def datetime_makedate(module, year, month, day):
 class ConfirmType(FancyValidator):
 
     """
-    Confirms that the input/output is of the proper type, using:
+    Confirms that the input/output is of the proper type.
+
+    Uses the parameters:
 
     subclass:
         The class or a tuple of classes; the item must be an instance
@@ -170,11 +175,12 @@ class ConfirmType(FancyValidator):
 class Wrapper(FancyValidator):
 
     """
-    Used to convert functions to validator/converters.  You can give a
-    simple function for `to_python`, `from_python`, `validate_python` or
-    `validate_other`.  If that function raises an exception, the value
-    is considered invalid.  Whatever value the function returns is
-    considered the converted value.
+    Used to convert functions to validator/converters.
+
+    You can give a simple function for `to_python`, `from_python`,
+    `validate_python` or `validate_other`.  If that function raises an
+    exception, the value is considered invalid.  Whatever value the
+    function returns is considered the converted value.
 
     Unlike validators, the `state` argument is not used.  Functions
     like `int` can be used here, that take a single argument.
@@ -234,16 +240,20 @@ class Wrapper(FancyValidator):
 class Constant(FancyValidator):
 
     """
-    This converter converts everything to the same thing.  I.e., you
-    pass in the constant value when initializing, then all values get
-    converted to that constant value.
+    This converter converts everything to the same thing.
+
+    I.e., you pass in the constant value when initializing, then all
+    values get converted to that constant value.
+
+    This is only really useful for funny situations, like::
 
-    This is only really useful for funny situations, like:
       fromEmailValidator = ValidateAny(
                                ValidEmailAddress(),
                                Constant('unknown@localhost'))
-    In this case, the if the email is not valid 'unknown@localhost' will
-    be used instead.  Of course, you could use if_invalid instead.
+                               
+    In this case, the if the email is not valid
+    ``'unknown@localhost'`` will be used instead.  Of course, you
+    could use ``if_invalid`` instead.
 
     Examples::
 
@@ -368,9 +378,10 @@ class MinLength(FancyValidator):
 class NotEmpty(FancyValidator):
 
     """
-    Invalid if value is empty (empty string, empty list, etc).  Generally
-    for objects that Python considers false, except zero which is not
-    considered invalid.
+    Invalid if value is empty (empty string, empty list, etc).
+
+    Generally for objects that Python considers false, except zero
+    which is not considered invalid.
 
     Examples::
 
@@ -421,6 +432,7 @@ class Regex(FancyValidator):
 
     """
     Invalid if the value doesn't match the regular expression `regex`.
+
     The regular expression can be a compiled re object, or a string
     which will be compiled for you.
 
@@ -513,12 +525,13 @@ class PlainText(Regex):
 class OneOf(FancyValidator):
 
     """
-    Tests that the value is one of the members of a given list.  If
-    testValueLists=True, then if the input value is a list or tuple,
-    all the members of the sequence will be checked (i.e., the input
-    must be a subset of the allowed values).
+    Tests that the value is one of the members of a given list.
 
-    Use hideList=True to keep the list of valid values out of the
+    If ``testValueLists=True``, then if the input value is a list or
+    tuple, all the members of the sequence will be checked (i.e., the
+    input must be a subset of the allowed values).
+
+    Use ``hideList=True`` to keep the list of valid values out of the
     error message in exceptions.
 
     Examples::
@@ -569,9 +582,10 @@ class DictConverter(FancyValidator):
 
     """
     Converts values based on a dictionary which has values as keys for
-    the resultant values.  If allowNull is passed, it will not balk if
-    a false value (e.g., '' or None) is given (it will return None in
-    these cases).
+    the resultant values.
+
+    If ``allowNull`` is passed, it will not balk if a false value
+    (e.g., '' or None) is given (it will return None in these cases).
 
     to_python takes keys and gives values, from_python takes values and
     gives keys.
@@ -826,8 +840,10 @@ class Number(FancyValidator):
 class String(FancyValidator):
     """
     Converts things to string, but treats empty things as the empty
-    string.  Also takes a `max` and `min` argument, and the string
-    length must fall in that range.
+    string.
+
+    Also takes a `max` and `min` argument, and the string length must
+    fall in that range.
     """
 
     min = None
@@ -839,11 +855,13 @@ class String(FancyValidator):
         }
     
     def validate_python(self, value, state):
-        if self.max is not None and len(value) > self.max:
+        if (self.max is not None and value is not None
+            and len(value) > self.max):
             raise Invalid(self.message('tooLong', state,
                                        max=self.max),
                           value, state)
-        if self.min is not None and len(value) < self.min:
+        if (self.min is not None
+            and (not value or len(value) < self.min)):
             raise Invalid(self.message('tooShort', state,
                                        min=self.min),
                           value, state)
@@ -859,11 +877,14 @@ class Set(FancyValidator):
 
     """
     This is for when you think you may return multiple values for a
-    certain field.  This way the result will always be a list, even if
-    there's only one result.  It's equivalent to
-    ForEach(convertToList=True).
+    certain field.
+
+    This way the result will always be a list, even if there's only
+    one result.  It's equivalent to ForEach(convertToList=True).
     """
 
+    if_empty = ()
+
     def _to_python(self, value, state):
         if isinstance(value, (list, tuple)):
             return value
@@ -873,10 +894,13 @@ class Set(FancyValidator):
             return [value]
 
 class Email(FancyValidator):
-    """Validate an email address.  If you pass resolve_domain=True,
-    then it will try to resolve the domain name to make sure it's valid.
-    This takes longer, of course.  You must have the pyDNS modules
-    installed <http://pydns.sf.net> to look up MX records.
+    """
+    Validate an email address.
+
+    If you pass ``resolve_domain=True``, then it will try to resolve
+    the domain name to make sure it's valid.  This takes longer, of
+    course.  You must have the `pyDNS <http://pydns.sf.net>`_ modules
+    installed to look up MX records.
     """
 
     resolve_domain = False
@@ -1022,11 +1046,12 @@ class URL(FancyValidator):
 class StateProvince(FancyValidator):
     
     """
-    Valid state or province code (two-letter).  Well, for now I don't
-    know the province codes, but it does state codes.  Give your own
-    `states` list to validate other state-like codes; give
-    `extraStates` to add values without losing the current state
-    values.
+    Valid state or province code (two-letter).
+
+    Well, for now I don't know the province codes, but it does state
+    codes.  Give your own `states` list to validate other state-like
+    codes; give `extraStates` to add values without losing the current
+    state values.
     """
 
     states = ['AK', 'AL', 'AR', 'AZ', 'CA', 'CO', 'CT', 'DC', 'DE',
@@ -1052,7 +1077,7 @@ class StateProvince(FancyValidator):
             raise Invalid(
                 self.message('empty', state),
                 value, state)
-        if len(value) != 2:
+        if not value or len(value) != 2:
             raise Invalid(
                 self.message('wrongLength', state),
                 value, state)
@@ -1070,6 +1095,7 @@ class PhoneNumber(FancyValidator):
     """
     Validates, and converts to ###-###-####, optionally with
     extension (as ext.##...)
+    
     @@: should add international phone number support
     """
 
@@ -1099,15 +1125,30 @@ class PhoneNumber(FancyValidator):
             result = result + " ext.%s" % match.group(4)
         return result
 
+class FieldStorageUploadConverter(FancyValidator):
+
+    """
+    Converts a cgi.FieldStorage instance to
+    a value that FormEncode can use for file
+    uploads.
+    """
+    def _to_python(self, value, state):
+        if isinstance(value, cgi.FieldStorage):
+            return fieldstorage.convert_fieldstorage(value)
+        else:
+            return value
+
 class DateConverter(FancyValidator):
 
     """
     Validates and converts a textual date, like mm/yy, dd/mm/yy,
-    dd-mm-yy, etc Always assumes month comes second value is the
-    month.  Accepts English month names, also abbreviated.  Returns
-    value as mx.DateTime object.  Two year dates are assumed to be
-    within 1950-2020, with dates from 21-49 being ambiguous and
-    signaling an error.
+    dd-mm-yy, etc, always assumes month comes second value is the
+    month.
+
+    Accepts English month names, also abbreviated.  Returns value as
+    mx.DateTime object.  Two year dates are assumed to be within
+    1950-2020, with dates from 21-49 being ambiguous and signaling an
+    error.
 
     Use accept_day=False if you just want a month/year (like for a
     credit card expiration date).
@@ -1239,6 +1280,8 @@ class DateConverter(FancyValidator):
         return datetime_makedate(dt_mod, year, month, 1)
 
     def _from_python(self, value, state):
+        if self.if_empty is not NoDefault and not value:
+            return ''
         if self.accept_day:
             return self.unconvert_day(value, state)
         else:
@@ -1451,8 +1494,10 @@ class StripField(FancyValidator):
 
     """
     Take a field from a dictionary, removing the key from the
-    dictionary.  ``name`` is the key.  The field value and a new copy
-    of the dictionary with that field removed are returned.
+    dictionary.
+
+    ``name`` is the key.  The field value and a new copy of the
+    dictionary with that field removed are returned.
     """
 
     __unpackargs__ = ('name',)
@@ -1472,6 +1517,101 @@ class StripField(FancyValidator):
                           valueDict, state)
         return field, v
 
+
+class StringBoolean(FancyValidator):
+    # Originally from TurboGears
+    """
+    Converts a string to a boolean.
+    
+    Values like 'true' and 'false' are considered True and False,
+    respectively; anything in ``true_values`` is true, anything in
+    ``false_values`` is false, case-insensitive).  The first item of
+    those lists is considered the preferred form.
+    """
+    
+    true_values = ['true', 't', 'yes', 'y', 'on', '1']
+    false_values = ['false', 'f', 'no', 'n', 'off', '0']
+
+    messages = { "string" : "Value should be %(true)r or %(false)r" }
+    
+    def _to_python(self, value, state):
+        if isinstance(value, (str, unicode)):
+            value = value.strip().lower()
+            if value in self.true_values:
+                return True
+            if not value or value in self.false_values:
+                return False
+            raise Invalid(self.message("string", state,
+                                       true=self.true_values[0],
+                                       false=self.false_values[0]),
+                          value, state)
+        return bool(value)
+    
+    def _from_python(self, value, state):
+        if value:
+            return self.true_values[0]
+        else:
+            return self.false_values[0]
+
+class SignedString(FancyValidator):
+
+    """
+    Encodes a string into a signed string, and base64 encodes both the
+    signature string and a random nonce.
+
+    It is up to you to provide a secret, and to keep the secret handy
+    and consistent.
+    """
+
+    messages = {
+        'malformed': 'Value does not contain a signature',
+        'badsig': 'Signature is not correct',
+        }
+
+    secret = None
+    nonce_length = 4
+
+    def _to_python(self, value, state):
+        global sha
+        if not sha:
+            import sha
+        assert secret, (
+            "You must give a secret")
+        parts = value.split(None, 1)
+        if not parts or len(parts) == 1:
+            raise Invalid(self.message('malformed', state),
+                          value, state)
+        sig, rest = parts
+        sig = sig.decode('base64')
+        rest = sig.decord('base64')
+        nonce = rest[:self.nonce_length]
+        rest = rest[self.nonce_length:]
+        digest = sha.new(self.secret+nonce+rest).digest()
+        if digest != sig:
+            raise Invalid(self.message('badsig', state),
+                          value, state)
+        return rest
+
+    def _from_python(self, value, state):
+        global sha
+        if not sha:
+            import sha
+        nonce = self.make_nonce()
+        value = str(value)
+        digest = sha.new(self.secret+nonce+rest).digest()
+        return self.encode(digest)+' '+self.encode(nonce+value)
+
+    def encode(self, value):
+        return value.encode('base64').strip().replace('\n', '')
+
+    def make_nonce(self):
+        global random
+        if not random:
+            import random
+        return ''.join([
+            chr(random.randrange(256))
+            for i in range(self.nonce_length)])
+
 class FormValidator(FancyValidator):
     """
     A FormValidator is something that can be chained with a
@@ -1538,7 +1678,7 @@ class FieldsMatch(FormValidator):
             error_message = '<br>\n'.join(
                 ['%s: %s' % (name, value) for name, value in error_list])
             raise Invalid(error_message,
-                          {}, field_dict, state,
+                          field_dict, state,
                           error_dict=errors)
 
 class CreditCardValidator(FormValidator):
@@ -1561,7 +1701,7 @@ class CreditCardValidator(FormValidator):
     __unpackargs__ = ('cc_type_field', 'cc_number_field')
 
     messages = {
-        'invalidNumber': "Please enter only the number, no other characters",
+        'notANumber': "Please enter only the number, no other characters",
         'badLength': "You did not enter a valid number of digits",
         'invalidNumber': "That number is not valid",
         }
@@ -1570,9 +1710,9 @@ class CreditCardValidator(FormValidator):
         if not field_dict.get(self.cc_type_field, None) \
            or not field_dict.get(self.cc_number_field, None):
             return None
-        self.validate(field_dict, state)
+        self.validate_python(field_dict, state)
 
-    def validate(self, field_dict, state):
+    def validate_python(self, field_dict, state):
         errors = self._validateReturn(field_dict, state)
         if errors:
             error_list = errors.items()
@@ -1580,7 +1720,6 @@ class CreditCardValidator(FormValidator):
             raise Invalid(
                 '<br>\n'.join(["%s: %s" % (name, value)
                                for name, value in error_list]),
-                {}, 
                 field_dict, state, error_dict=errors)
         
     def _validateReturn(self, field_dict, state):
@@ -1591,7 +1730,7 @@ class CreditCardValidator(FormValidator):
         try:
             long(number)
         except ValueError:
-            return {self.cc_number_field: self.message('invalidNumber', state)}
+            return {self.cc_number_field: self.message('notANumber', state)}
 
         assert self._cardInfo.has_key(ccType), (
             "I can't validate that type of credit card")
@@ -1600,8 +1739,8 @@ class CreditCardValidator(FormValidator):
         for prefix, length in self._cardInfo[ccType]:
             if len(number) == length:
                 validLength = True
-            if len(number) == length \
-               and number[:len(prefix)] != prefix:
+            if (len(number) == length 
+                and number.startswith(prefix)):
                 foundValid = True
                 break
         if not validLength:
